@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,20 +21,11 @@
 
 package com.openexchange.messaging.generic.groupware;
 
-import java.sql.Connection;
-import java.sql.DatabaseMetaData;
-import java.sql.PreparedStatement;
-import java.sql.ResultSet;
-import java.sql.SQLException;
 import com.openexchange.database.AbstractCreateTableImpl;
-import com.openexchange.database.Databases;
-import com.openexchange.exception.OXException;
 import com.openexchange.groupware.update.Attributes;
 import com.openexchange.groupware.update.PerformParameters;
 import com.openexchange.groupware.update.TaskAttributes;
-import com.openexchange.groupware.update.UpdateExceptionCodes;
 import com.openexchange.groupware.update.UpdateTaskV2;
-import com.openexchange.messaging.generic.services.MessagingGenericServiceRegistry;
 
 /**
  * {@link MessagingGenericCreateTableTask}
@@ -94,30 +85,4 @@ public final class MessagingGenericCreateTableTask extends AbstractCreateTableIm
         return new String[] { "messagingAccount" };
     }
 
-    private void createTable(String tablename, String sqlCreate, Connection connection) throws OXException {
-        PreparedStatement stmt = null;
-        try {
-            if (tableExists(connection, tablename)) {
-                return;
-            }
-
-            stmt = connection.prepareStatement(sqlCreate);
-            stmt.executeUpdate();
-        } catch (SQLException e) {
-            throw UpdateExceptionCodes.SQL_PROBLEM.create(e, e.getMessage());
-        } finally {
-            Databases.closeSQLStuff(stmt);
-        }
-    }
-
-    private boolean tableExists(final Connection con, final String table) throws SQLException {
-        final DatabaseMetaData metaData = con.getMetaData();
-        ResultSet rs = null;
-        try {
-            rs = metaData.getTables(null, null, table, new String[] { "TABLE" });
-            return (rs.next() && rs.getString("TABLE_NAME").equals(table));
-        } finally {
-            Databases.closeSQLStuff(rs);
-        }
-    }
 }
