@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -65,8 +65,9 @@ public class Enabled implements PreferencesItemService {
             public void getValue(final Session session, final Context ctx, final User user, final UserConfiguration userConfig, final Setting setting) throws OXException {
                     final ConfigView view = configViews.getView(user.getId(), ctx.getContextId());
                     final ComposedConfigProperty<Boolean> property = view.property(ENABLED, boolean.class);
-                    if (property.isDefined()) {
-                        setting.setSingleValue(property.get());
+                    Boolean value = property.get();
+                    if (value != null) {
+                        setting.setSingleValue(value);
                     } else {
                         setting.setSingleValue(Boolean.TRUE);
                     }
