@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -32,6 +32,7 @@ import org.json.JSONObject;
 import com.openexchange.ajax.requesthandler.AJAXRequestData;
 import com.openexchange.caching.Cache;
 import com.openexchange.exception.OXException;
+import com.openexchange.java.Strings;
 import com.openexchange.messaging.MessagingAccountAccess;
 import com.openexchange.messaging.MessagingAccountTransport;
 import com.openexchange.messaging.MessagingAddressHeader;
@@ -195,7 +196,7 @@ public class MessagingRequestData {
             return new MessagingField[0];
         }
 
-        final String[] columnList = parameter.split("\\s*,\\s*");
+        final String[] columnList = Strings.splitByComma(parameter);
         final MessagingField[] fields = MessagingField.getFields(columnList);
         for (int i = 0; i < fields.length; i++) {
             if (fields[i] == null) {
