@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 
 package com.openexchange.rss.utils;
 
+import java.net.URI;
 import com.openexchange.osgi.annotation.SingletonService;
 
 /**
@@ -84,7 +85,7 @@ public interface RssProperties {
     // -------------------------------------------------------------------------------------------------------------------------------------
 
     /**
-     * Checks if specified host name and port are denied to connect against.
+     * Checks if specified URI is denied to connect against.
      * <p>
      * The host name can either be a machine name, such as "<code>java.sun.com</code>", or a textual representation of its IP address.
      *
@@ -93,4 +94,14 @@ public interface RssProperties {
      */
     boolean isDenied(String uriString);
 
+    /**
+     * Checks if specified URI is denied to connect against.
+     *
+     * @param uri The URI of the RSS feed
+     * @return <code>true</code> if denied; otherwise <code>false</code>
+     */
+    default boolean isDenied(URI uri) {
+        return null == uri || isDenied(uri.toString());
+    }
+
 }
