@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -100,7 +100,7 @@ public class RssPropertiesImpl implements RssProperties {
 
     @Override
     public boolean isBlacklisted(String hostName) {
-        return Strings.isEmpty(hostName) ? false : blacklistedHosts().contains(hostName.trim());
+        return Strings.isNotEmpty(hostName) && blacklistedHosts().contains(hostName.trim());
     }
 
     // ---------------------------------------- Allowed ports ------------------------------------------------------------------------------
@@ -250,33 +250,40 @@ public class RssPropertiesImpl implements RssProperties {
 
     @Override
     public boolean isDenied(String uriString) {
-        URI uri;
         try {
-            uri = new URI(uriString);
-            int port = uri.getPort();
-            if (port < 0) {
-                String scheme = Strings.asciiLowerCase(uri.getScheme());
-                if (Strings.isEmpty(scheme)) {
-                    // Assume HTTP as default
-                    port = 80;
-                } else {
-                    scheme = scheme.trim();
-                    if ("http".equals(scheme)) {
-                        port = 80;
-                    } else if ("https".equals(scheme)) {
-                        port = 443;
-                    }  else {
-                        port = 80;
-                    }
-                }
-            }
-            return !isAllowed(port) || isBlacklisted(uri.getHost()) || !isAllowedScheme(uri.getScheme()) || !isValid(uri);
+            return isDenied(new URI(uriString));
         } catch (URISyntaxException e) {
             LoggerHolder.LOG.debug("Given feed URL \"{}\" appears not to be valid.", uriString, e);
             return true;
         }
     }
 
+    @Override
+    public boolean isDenied(URI uri) {
+        if (uri == null) {
+            return true;
+        }
+
+        int port = uri.getPort();
+        if (port < 0) {
+            String scheme = Strings.asciiLowerCase(uri.getScheme());
+            if (Strings.isEmpty(scheme)) {
+                // Assume HTTP as default
+                port = 80;
+            } else {
+                scheme = scheme.trim();
+                if ("http".equals(scheme)) {
+                    port = 80;
+                } else if ("https".equals(scheme)) {
+                    port = 443;
+                }  else {
+                    port = 80;
+                }
+            }
+        }
+        return !isAllowed(port) || isBlacklisted(uri.getHost()) || !isAllowedScheme(uri.getScheme()) || !isValid(uri);
+    }
+
     /**
      * Checks whether the given {@link URI} is valid or not
      *
