@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -100,7 +100,7 @@ public class RssAction implements AJAXActionService {
     // ------------------------------------------------------------------------------------------------------------------------------
 
     private final TimeoutHttpURLFeedFetcher fetcher;
-    private final HashMapFeedInfoCache     feedCache;
+    private final HashMapFeedInfoCache feedCache;
 
     /**
      * Initializes a new {@link RssAction}.
@@ -168,15 +168,15 @@ public class RssAction implements AJAXActionService {
                     if (imageUrl != null) {
                         result.setImageUrl(imageUrl);
                     }
+
+                    // Add to results list
+                    results.add(result);
+
+                    @SuppressWarnings("unchecked") List<SyndContent> contents = entry.getContents();
+                    handleContents(preprocessor, entry, result, contents);
                 } catch (MalformedURLException e) {
-                    throw RssExceptionCodes.INVALID_RSS.create(e, entry.getLink());
+                    LOG.debug("Unable to process entry with invalid link {}", entry.getLink(), e);
                 }
-
-                // Add to results list
-                results.add(result);
-
-                @SuppressWarnings("unchecked") List<SyndContent> contents = entry.getContents();
-                handleContents(preprocessor, entry, result, contents);
             }
         }
 
