@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -30,22 +30,37 @@ import com.openexchange.capabilities.CapabilityService;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.config.cascade.ConfigView;
 import com.openexchange.config.cascade.ConfigViewFactory;
+import com.openexchange.config.lean.LeanConfigurationService;
 import com.openexchange.exception.OXException;
 import com.openexchange.html.HtmlService;
 import com.openexchange.net.ssl.SSLSocketFactoryProvider;
 import com.openexchange.net.ssl.config.UserAwareSSLConfigurationService;
+import com.openexchange.rest.client.httpclient.HttpClientService;
+import com.openexchange.rest.client.httpclient.SpecificHttpClientConfigProvider;
 import com.openexchange.rss.RssJsonConverter;
 import com.openexchange.rss.actions.RssActionFactory;
+import com.openexchange.rss.httpclient.properties.RssFeedHttpProperties;
 import com.openexchange.rss.utils.RssProperties;
 import com.openexchange.session.Session;
 import com.openexchange.tools.session.ServerSession;
 import com.openexchange.tools.session.ServerSessionAdapter;
 
+/**
+ * Activator for <code>"com.openexchange.rss"</code> bundle.
+ */
 public class RssActivator extends AJAXModuleActivator {
 
+    /**
+     * Initializes a new {@link RssActivator}.
+     */
+    public RssActivator() {
+        super();
+    }
+
     @Override
     protected Class<?>[] getNeededServices() {
-        return new Class<?>[] { HtmlService.class, CapabilityService.class, ConfigViewFactory.class, ConfigurationService.class, SSLSocketFactoryProvider.class, UserAwareSSLConfigurationService.class, RssProperties.class };
+        return new Class<?>[] { HtmlService.class, CapabilityService.class, ConfigViewFactory.class, ConfigurationService.class,
+            SSLSocketFactoryProvider.class, UserAwareSSLConfigurationService.class, RssProperties.class, HttpClientService.class, LeanConfigurationService.class };
     }
 
     @Override
@@ -54,29 +69,33 @@ public class RssActivator extends AJAXModuleActivator {
         registerModule(new RssActionFactory(), "rss");
         registerService(ResultConverter.class, new RssJsonConverter());
 
-        final Dictionary<String, Object> properties = new Hashtable<String, Object>(1);
-        properties.put(CapabilityChecker.PROPERTY_CAPABILITIES, "rss");
-        registerService(CapabilityChecker.class, new CapabilityChecker() {
+        {
+            final Dictionary<String, Object> properties = new Hashtable<String, Object>(1);
+            properties.put(CapabilityChecker.PROPERTY_CAPABILITIES, "rss");
+            registerService(CapabilityChecker.class, new CapabilityChecker() {
 
-            @Override
-            public boolean isEnabled(String capability, Session ses) throws OXException {
-                if ("rss".equals(capability)) {
-                    final ServerSession session = ServerSessionAdapter.valueOf(ses);
-                    if (session.isAnonymous() || session.getUser().isGuest()) {
-                        return false;
-                    }
-                    final ConfigViewFactory factory = getService(ConfigViewFactory.class);
-                    if (null != factory) {
-                        final ConfigView view = factory.getView(session.getUserId(), session.getContextId());
-                        return view.opt("com.openexchange.rss", boolean.class, Boolean.TRUE).booleanValue() && view.opt("com.openexchange.messaging.rss", boolean.class, Boolean.TRUE).booleanValue();
+                @Override
+                public boolean isEnabled(String capability, Session ses) throws OXException {
+                    if ("rss".equals(capability)) {
+                        final ServerSession session = ServerSessionAdapter.valueOf(ses);
+                        if (session.isAnonymous() || session.getUser().isGuest()) {
+                            return false;
+                        }
+                        final ConfigViewFactory factory = getService(ConfigViewFactory.class);
+                        if (null != factory) {
+                            final ConfigView view = factory.getView(session.getUserId(), session.getContextId());
+                            return view.opt("com.openexchange.rss", boolean.class, Boolean.TRUE).booleanValue() && view.opt("com.openexchange.messaging.rss", boolean.class, Boolean.TRUE).booleanValue();
+                        }
                     }
+
+                    return true;
                 }
-
-                return true;
-            }
-        }, properties);
+            }, properties);
+        }
 
         getService(CapabilityService.class).declareCapability("rss");
+
+        registerService(SpecificHttpClientConfigProvider.class, new RssFeedHttpProperties(this));
     }
 
     @Override
