@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,23 +21,24 @@
 
 package com.openexchange.rss.util;
 
-import static com.openexchange.java.Autoboxing.I;
+import static com.openexchange.rss.httpclient.properties.RssFeedHttpProperties.getHttpClientId;
 import java.io.BufferedInputStream;
 import java.io.IOException;
 import java.io.InputStream;
-import java.net.HttpURLConnection;
 import java.net.InetAddress;
-import java.net.MalformedURLException;
-import java.net.URI;
-import java.net.URISyntaxException;
 import java.net.URL;
-import java.net.URLConnection;
 import java.net.UnknownHostException;
-import java.util.Set;
-import java.util.zip.GZIPInputStream;
-import javax.net.ssl.HttpsURLConnection;
-import javax.net.ssl.SSLSocketFactory;
-import com.google.common.collect.ImmutableSet;
+import java.util.Date;
+import org.apache.http.Header;
+import org.apache.http.HttpEntity;
+import org.apache.http.HttpHeaders;
+import org.apache.http.HttpResponse;
+import org.apache.http.HttpStatus;
+import org.apache.http.client.HttpClient;
+import org.apache.http.client.methods.HttpGet;
+import org.apache.http.client.utils.DateUtils;
+import org.apache.http.protocol.BasicHttpContext;
+import org.apache.http.protocol.HttpContext;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.config.Interests;
 import com.openexchange.config.Reloadable;
@@ -45,9 +46,10 @@ import com.openexchange.config.Reloadables;
 import com.openexchange.java.InetAddresses;
 import com.openexchange.java.Streams;
 import com.openexchange.java.Strings;
-import com.openexchange.net.ssl.SSLSocketFactoryProvider;
+import com.openexchange.rest.client.httpclient.HttpClientService;
+import com.openexchange.rest.client.httpclient.HttpClients;
+import com.openexchange.rss.httpclient.properties.RssFeedRedirectStrategy;
 import com.openexchange.rss.osgi.Services;
-import com.openexchange.rss.utils.RssProperties;
 import com.openexchange.tools.stream.CountingInputStream;
 import com.sun.syndication.feed.synd.SyndFeed;
 import com.sun.syndication.fetcher.FetcherEvent;
@@ -113,55 +115,12 @@ public class TimeoutHttpURLFeedFetcher extends AbstractFeedFetcher implements Re
         reloadConfiguration(Services.getService(ConfigurationService.class));
     }
 
-    private static final Set<Integer> REDIRECT_RESPONSE_CODES = ImmutableSet.of(I(HttpURLConnection.HTTP_MOVED_PERM), I(HttpURLConnection.HTTP_MOVED_TEMP), I(HttpURLConnection.HTTP_SEE_OTHER), I(HttpURLConnection.HTTP_USE_PROXY));
-
-    private URL checkPossibleRedirect(int responseCode, HttpURLConnection httpConnection, boolean originalAddressIsRemote) throws FetcherException {
-        if (!REDIRECT_RESPONSE_CODES.contains(I(responseCode))) {
-            // No redirect response code
-            return null;
-        }
-
-        // Get redirect location & disconnect current URL connection instance
-        String redirectUrl = httpConnection.getHeaderField("Location");
-        httpConnection.disconnect();
-
-        // Validate redirect location
-        if (Strings.isEmpty(redirectUrl)) {
-            // Missing "Location" header
-            throw new FetcherException("Missing redirect URL");
-        }
-
-        // Acquire needed service
-        RssProperties rssProperties = Services.getService(RssProperties.class);
-        if (rssProperties == null) {
-            throw new FetcherException("Missing service: " + RssProperties.class.getName());
-        }
-
-        // Examine redirect location
-        try {
-            if (rssProperties.isDenied(redirectUrl)) {
-                // Deny redirecting to a local address
-                throw new FetcherException(redirectUrl + " is not an allowed redirect URL");
-            }
-            URI feedUrl = new URI(redirectUrl);
-            if (originalAddressIsRemote) {
-                try {
-                    InetAddress inetAddress = InetAddress.getByName(feedUrl.getHost());
-                    if (InetAddresses.isInternalAddress(inetAddress)) {
-                        // Deny redirecting to a local address
-                        throw new FetcherException(feedUrl.toString() + " is not an allowed redirect URL");
-                    }
-                } catch (UnknownHostException e) {
-                    // IP address of that host could not be determined
-                    LOG.warn("Unknown host: {}. Skipping retrieving feed from redirect URL {}", feedUrl.getHost(), feedUrl, e);
-                    throw new IllegalArgumentException(feedUrl.toString() + " contains an unknown host", e);
-                }
-            }
-            return feedUrl.toURL();
-        } catch (URISyntaxException | MalformedURLException e) {
-            LOG.warn("Redirect URL is invalid: {}", redirectUrl, e);
-            throw new IllegalArgumentException("Redirect URL is invalid: " + redirectUrl, e);
+    private HttpClient getHttpClient() throws FetcherException {
+        HttpClientService httpClientService = Services.optService(HttpClientService.class);
+        if (httpClientService == null) {
+            throw new FetcherException("Absent service: " + HttpClientService.class.getName());
         }
+        return httpClientService.getHttpClient(getHttpClientId());
     }
 
     /**
@@ -190,92 +149,81 @@ public class TimeoutHttpURLFeedFetcher extends AbstractFeedFetcher implements Re
             throw new IllegalArgumentException(feedUrlToRetrieve.toExternalForm() + " contains an unknown host", e);
         }
 
-        URL feedUrl = feedUrlToRetrieve;
-        NextUrl: while (true) {
-            URLConnection connection = feedUrl.openConnection();
-            if (!(connection instanceof HttpURLConnection)) {
-                throw new IllegalArgumentException(feedUrl.toExternalForm() + " is not a valid HTTP Url");
+        HttpContext httpContext = new BasicHttpContext();
+        httpContext.setAttribute(RssFeedRedirectStrategy.ATTRIBUTE_ORIGINAL_ADDR_IS_REMOTE, Boolean.valueOf(originalAddressIsRemote));
+
+        FeedFetcherCache cache = getFeedInfoCache();
+        SyndFeedInfo syndFeedInfo = cache == null ? null : cache.getFeedInfo(feedUrlToRetrieve);
+
+        HttpGet request = null;
+        HttpResponse httpResponse = null;
+        try {
+            // Execute HTTP request
+            request = new HttpGet(feedUrlToRetrieve.toString());
+            setRequestHeaders(request, syndFeedInfo);
+            httpResponse = getHttpClient().execute(request, httpContext);
+            fireEvent(FetcherEvent.EVENT_TYPE_FEED_POLLED, feedUrlToRetrieve.toExternalForm());
+
+            // Check status code
+            int statusCode = httpResponse.getStatusLine().getStatusCode();
+            switch (statusCode) {
+                case HttpStatus.SC_UNAUTHORIZED:
+                    throw new FetcherException("RSS feed \"" + feedUrlToRetrieve + "\" requires authentication");
+                case HttpStatus.SC_NOT_FOUND:
+                    throw new FetcherException("RSS feed \"" + feedUrlToRetrieve + "\" not found");
+                case HttpStatus.SC_SERVICE_UNAVAILABLE:
+                    throw new FetcherException("RSS feed \"" + feedUrlToRetrieve + "\" has been removed");
+            }
+            if (statusCode >= 400 && statusCode <= 499) {
+                String reasonPhrase = httpResponse.getStatusLine().getReasonPhrase();
+                throw new FetcherException("RSS feed \"" + feedUrlToRetrieve + "\" responded with unexpected error" + (Strings.isEmpty(reasonPhrase) ? "" : ": " + reasonPhrase));
+            }
+            if (statusCode >= 500 && statusCode <= 599) {
+                String reasonPhrase = httpResponse.getStatusLine().getReasonPhrase();
+                throw new FetcherException("RSS feed \"" + feedUrlToRetrieve + "\" responded with remote server error" + (Strings.isEmpty(reasonPhrase) ? "" : ": " + reasonPhrase));
+            }
+            handleErrorCodes(statusCode);
+
+            if (syndFeedInfo != null && HttpStatus.SC_NOT_MODIFIED == statusCode) {
+                // the feed does not need retrieving
+                fireEvent(FetcherEvent.EVENT_TYPE_FEED_UNCHANGED, feedUrlToRetrieve.toExternalForm());
+                return syndFeedInfo.getSyndFeed();
             }
 
-            HttpURLConnection httpConnection = (HttpURLConnection) connection;
-            if ("https".equalsIgnoreCase(feedUrl.getProtocol()) || 443 == feedUrl.getPort()) {
-                SSLSocketFactory sslSocketFactory = Services.getService(SSLSocketFactoryProvider.class).getDefault();
-                ((HttpsURLConnection) httpConnection).setSSLSocketFactory(sslSocketFactory);
+            HttpEntity entity = httpResponse.getEntity();
+            if (entity == null) {
+                throw new FetcherException("RSS feed \"" + feedUrlToRetrieve + "\" provides no content");
             }
 
-            if (connectTimout > 0) {
-                httpConnection.setConnectTimeout(connectTimout);
-            }
-            if (readTimout > 0) {
-                httpConnection.setReadTimeout(readTimout);
+            if (maximumAllowedSize > 0) {
+                long contentLength = entity.getContentLength();
+                Header contentLengthHeader = httpResponse.getFirstHeader("Content-Length");
+                String contentLength2 = contentLengthHeader == null ? null : contentLengthHeader.getValue();
+
+                long allowedFeedSize = maximumAllowedSize;
+                if (contentLength > allowedFeedSize || (Strings.isNotEmpty(contentLength2) && Long.parseLong(contentLength2) > allowedFeedSize)) {
+                    throw new FetcherException("RSS feed \"" + feedUrlToRetrieve + "\" is too big");
+                }
             }
 
-            // Deny to automatically follow redirects
-            httpConnection.setInstanceFollowRedirects(false);
-
-            FeedFetcherCache cache = getFeedInfoCache();
             if (cache == null) {
-                fireEvent(FetcherEvent.EVENT_TYPE_FEED_POLLED, connection);
                 InputStream inputStream = null;
-                setRequestHeaders(connection, null);
-                httpConnection.connect();
                 try {
-                    // Check for redirect
-                    int responseCode = httpConnection.getResponseCode();
-                    URL redirectUrl = checkPossibleRedirect(responseCode, httpConnection, originalAddressIsRemote);
-                    if (null != redirectUrl) {
-                        feedUrl = redirectUrl;
-                        continue NextUrl;
-                    }
-
-                    inputStream = httpConnection.getInputStream();
-                    return getSyndFeedFromStream(inputStream, connection);
-                } catch (@SuppressWarnings("unused") java.io.IOException e) {
-                    handleErrorCodes(((HttpURLConnection) connection).getResponseCode());
+                    inputStream = entity.getContent();
+                    return getSyndFeedFromStream(inputStream, entity, feedUrlToRetrieve);
                 } finally {
                     Streams.close(inputStream);
-                    httpConnection.disconnect();
                 }
-                // we will never actually get to this line
-                return null;
             }
 
-            // With cache
-            SyndFeedInfo syndFeedInfo = cache.getFeedInfo(feedUrl);
-            setRequestHeaders(connection, syndFeedInfo);
-            httpConnection.connect();
-            try {
-                fireEvent(FetcherEvent.EVENT_TYPE_FEED_POLLED, connection);
-
-                // Check for redirect
-                int responseCode = httpConnection.getResponseCode();
-                URL redirectUrl = checkPossibleRedirect(responseCode, httpConnection, originalAddressIsRemote);
-                if (null != redirectUrl) {
-                    feedUrl = redirectUrl;
-                    continue NextUrl;
-                }
-
-                if (syndFeedInfo == null) {
-                    // this is a feed that hasn't been retrieved
-                    syndFeedInfo = new SyndFeedInfo();
-                    retrieveAndCacheFeed(feedUrl, syndFeedInfo, httpConnection, responseCode);
-                } else {
-                    // check the response code
-                    if (responseCode != HttpURLConnection.HTTP_NOT_MODIFIED) {
-                        // the response code is not 304 NOT MODIFIED
-                        // This is either because the feed server
-                        // does not support condition gets
-                        // or because the feed hasn't changed
-                        retrieveAndCacheFeed(feedUrl, syndFeedInfo, httpConnection, responseCode);
-                    } else {
-                        // the feed does not need retrieving
-                        fireEvent(FetcherEvent.EVENT_TYPE_FEED_UNCHANGED, connection);
-                    }
-                }
-                return syndFeedInfo.getSyndFeed();
-            } finally {
-                httpConnection.disconnect();
+            if (syndFeedInfo == null) {
+                // this is a feed that hasn't been retrieved
+                syndFeedInfo = new SyndFeedInfo();
             }
+            retrieveAndCacheFeed(feedUrlToRetrieve, syndFeedInfo, entity, httpResponse);
+            return syndFeedInfo.getSyndFeed();
+        } finally {
+            HttpClients.close(request, httpResponse);
         }
     }
 
@@ -284,16 +232,14 @@ public class TimeoutHttpURLFeedFetcher extends AbstractFeedFetcher implements Re
      *
      * @param feedUrl The feed {@link URL}
      * @param syndFeedInfo The {@link SyndFeedInfo} to retrieve and cache
-     * @param connection The {@link HttpURLConnection}
+     * @param entity The HTTP entity
+     * @param httpResponse The HTTP response
      * @throws IllegalArgumentException
      * @throws FeedException
-     * @throws FetcherException
      * @throws IOException if an I/O error occurs
      */
-    protected void retrieveAndCacheFeed(URL feedUrl, SyndFeedInfo syndFeedInfo, HttpURLConnection connection, int responseCode) throws IllegalArgumentException, FeedException, FetcherException, IOException {
-        handleErrorCodes(responseCode);
-
-        resetFeedInfo(feedUrl, syndFeedInfo, connection);
+    protected void retrieveAndCacheFeed(URL feedUrl, SyndFeedInfo syndFeedInfo, HttpEntity entity, HttpResponse httpResponse) throws IllegalArgumentException, FeedException, IOException {
+        resetFeedInfo(feedUrl, syndFeedInfo, entity, httpResponse);
         FeedFetcherCache cache = getFeedInfoCache();
         // resetting feed info in the cache
         // could be needed for some implementations
@@ -308,35 +254,38 @@ public class TimeoutHttpURLFeedFetcher extends AbstractFeedFetcher implements Re
      *
      * @param orignalUrl The original URL
      * @param syndFeedInfo The {@link SyndFeedInfo} to reset
-     * @param connection The {@link HttpURLConnection}
+     * @param entity The HTTP entity
+     * @param httpResponse The HTTP response
      * @throws IllegalArgumentException
      * @throws IOException if an I/O error occurs
      * @throws FeedException
      */
-    protected void resetFeedInfo(URL orignalUrl, SyndFeedInfo syndFeedInfo, HttpURLConnection connection) throws IllegalArgumentException, IOException, FeedException {
+    protected void resetFeedInfo(URL orignalUrl, SyndFeedInfo syndFeedInfo, HttpEntity entity, HttpResponse httpResponse) throws IllegalArgumentException, IOException, FeedException {
         // need to always set the URL because this may have changed due to 3xx redirects
-        syndFeedInfo.setUrl(connection.getURL());
+        syndFeedInfo.setUrl(orignalUrl);
 
         // the ID is a persistant value that should stay the same even if the URL for the
         // feed changes (eg, by 3xx redirects)
         syndFeedInfo.setId(orignalUrl.toString());
 
         // This will be 0 if the server doesn't support or isn't setting the last modified header
-        syndFeedInfo.setLastModified(new Long(connection.getLastModified()));
+        Header lastModifiedHeader = httpResponse.getFirstHeader(HttpHeaders.LAST_MODIFIED);
+        syndFeedInfo.setLastModified(lastModifiedHeader == null ? Long.valueOf(0) : Long.valueOf(DateUtils.parseDate(lastModifiedHeader.getValue()).getTime()));
 
         // This will be null if the server doesn't support or isn't setting the ETag header
-        syndFeedInfo.setETag(connection.getHeaderField("ETag"));
+        Header eTagHeader = httpResponse.getFirstHeader(HttpHeaders.ETAG);
+        syndFeedInfo.setETag(eTagHeader == null ? null : eTagHeader.getValue());
 
         // get the contents
         InputStream inputStream = null;
         try {
-            inputStream = connection.getInputStream();
-            SyndFeed syndFeed = getSyndFeedFromStream(inputStream, connection);
+            inputStream = entity.getContent();
+            SyndFeed syndFeed = getSyndFeedFromStream(inputStream, entity, orignalUrl);
 
-            String imHeader = connection.getHeaderField("IM");
-            if (isUsingDeltaEncoding() && (imHeader != null && imHeader.indexOf("feed") >= 0)) {
+            Header imHeader = httpResponse.getFirstHeader("IM");
+            if (isUsingDeltaEncoding() && (imHeader != null && imHeader.getValue().indexOf("feed") >= 0)) {
                 FeedFetcherCache cache = getFeedInfoCache();
-                if (cache != null && connection.getResponseCode() == 226) {
+                if (cache != null && httpResponse.getStatusLine().getStatusCode() == 226) {
                     // client is setup to use http delta encoding and the server supports it and has returned a delta encoded response
                     // This response only includes new items
                     SyndFeedInfo cachedInfo = cache.getFeedInfo(orignalUrl);
@@ -360,32 +309,35 @@ public class TimeoutHttpURLFeedFetcher extends AbstractFeedFetcher implements Re
     /**
      * <p>Set appropriate HTTP headers, including conditional get and gzip encoding headers</p>
      *
-     * @param connection A URLConnection
+     * @param request The HTTP request
      * @param syndFeedInfo The SyndFeedInfo for the feed to be retrieved. May be null
      */
-    protected void setRequestHeaders(URLConnection connection, SyndFeedInfo syndFeedInfo) {
+    protected void setRequestHeaders(HttpGet request, SyndFeedInfo syndFeedInfo) {
         if (syndFeedInfo != null) {
             // set the headers to get feed only if modified
             // we support the use of both last modified and eTag headers
             if (syndFeedInfo.getLastModified() != null) {
                 Object lastModified = syndFeedInfo.getLastModified();
                 if (lastModified instanceof Long) {
-                    connection.setIfModifiedSince(((Long) syndFeedInfo.getLastModified()).longValue());
+                    String ifModifiedSince = DateUtils.formatDate(new Date(((Long) syndFeedInfo.getLastModified()).longValue()));
+                    if (Strings.isNotEmpty(ifModifiedSince)) {
+                        request.setHeader(HttpHeaders.IF_MODIFIED_SINCE, ifModifiedSince);
+                    }
                 }
             }
             if (syndFeedInfo.getETag() != null) {
-                connection.setRequestProperty("If-None-Match", syndFeedInfo.getETag());
+                request.addHeader(HttpHeaders.IF_NONE_MATCH, syndFeedInfo.getETag());
             }
 
         }
         // header to retrieve feed gzipped
-        connection.setRequestProperty("Accept-Encoding", "gzip");
+        request.addHeader(HttpHeaders.ACCEPT_ENCODING, "gzip");
 
         // set the user agent
-        connection.addRequestProperty("User-Agent", getUserAgent());
+        request.addHeader("User-Agent", getUserAgent());
 
         if (isUsingDeltaEncoding()) {
-            connection.addRequestProperty("A-IM", "feed");
+            request.addHeader("A-IM", "feed");
         }
     }
 
@@ -393,15 +345,16 @@ public class TimeoutHttpURLFeedFetcher extends AbstractFeedFetcher implements Re
      * Reads the SyndFeed from the specified input stream and fires an event {@link FetcherEvent#EVENT_TYPE_FEED_RETRIEVED}
      *
      * @param inputStream The {@link InputStream}
-     * @param connection The {@link URLConnection}
+     * @param entity The HTTP entity
+     * @param feedUrl The URL of the RSS feed
      * @return The {@link SyndFeed}
      * @throws IOException If an I/O error occurs
      * @throws IllegalArgumentException
      * @throws FeedException
      */
-    private SyndFeed getSyndFeedFromStream(InputStream inputStream, URLConnection connection) throws IOException, IllegalArgumentException, FeedException {
-        SyndFeed feed = readSyndFeedFromStream(inputStream, connection);
-        fireEvent(FetcherEvent.EVENT_TYPE_FEED_RETRIEVED, connection, feed);
+    private SyndFeed getSyndFeedFromStream(InputStream inputStream, HttpEntity entity, URL feedUrl) throws IOException, IllegalArgumentException, FeedException {
+        SyndFeed feed = readSyndFeedFromStream(inputStream, entity);
+        fireEvent(FetcherEvent.EVENT_TYPE_FEED_RETRIEVED, feedUrl.toExternalForm(), feed);
         return feed;
     }
 
@@ -409,27 +362,21 @@ public class TimeoutHttpURLFeedFetcher extends AbstractFeedFetcher implements Re
      * Reads the SyndFeed from the specified input stream
      *
      * @param inputStream The {@link InputStream}
-     * @param connection The {@link URLConnection}
+     * @param entity The HTTP entity
      * @return The {@link SyndFeed}
      * @throws IOException If an I/O error occurs
      * @throws IllegalArgumentException
      * @throws FeedException
      */
-    private SyndFeed readSyndFeedFromStream(InputStream inputStream, URLConnection connection) throws IOException, IllegalArgumentException, FeedException {
-        CountingInputStream cis = inputStream instanceof CountingInputStream ? (CountingInputStream) inputStream : new CountingInputStream(inputStream, maximumAllowedSize);
-        InputStream is;
-        if ("gzip".equalsIgnoreCase(connection.getContentEncoding())) {
-            // handle gzip encoded content
-            is = new GZIPInputStream(cis, 65536);
-        } else {
-            is = new BufferedInputStream(cis, 65536);
-        }
-
-        XmlReader reader = null;
-        if (connection.getHeaderField("Content-Type") != null) {
-            reader = new XmlReader(is, connection.getHeaderField("Content-Type"), true);
-        } else {
-            reader = new XmlReader(is, true);
+    private SyndFeed readSyndFeedFromStream(InputStream inputStream, HttpEntity entity) throws IOException, IllegalArgumentException, FeedException {
+        XmlReader reader;
+        {
+            CountingInputStream cis = inputStream instanceof CountingInputStream ? (CountingInputStream) inputStream : new CountingInputStream(inputStream, maximumAllowedSize);
+            if (entity.getContentType() != null) {
+                reader = new XmlReader(new BufferedInputStream(cis, 65536), entity.getContentType().getValue(), true);
+            } else {
+                reader = new XmlReader(new BufferedInputStream(cis, 65536), true);
+            }
         }
 
         SyndFeedInput syndFeedInput = new SyndFeedInput();
