@@ -16,13 +16,11 @@
 
 package twitter4j.conf;
 
-import twitter4j.internal.util.T4JInternalStringUtil;
+import twitter4j.internal.util.z_T4JInternalStringUtil;
 
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.IOException;
-import java.io.InputStream;
+import java.io.*;
 import java.util.ArrayList;
+import java.util.Map;
 import java.util.Properties;
 import java.util.Set;
 
@@ -68,19 +66,21 @@ public final class PropertyConfiguration extends ConfigurationBase implements ja
     public static final String OAUTH_AUTHENTICATION_URL = "oauth.authenticationURL";
 
     public static final String REST_BASE_URL = "restBaseURL";
-    public static final String SEARCH_BASE_URL = "searchBaseURL";
     public static final String STREAM_BASE_URL = "streamBaseURL";
     public static final String USER_STREAM_BASE_URL = "userStreamBaseURL";
     public static final String SITE_STREAM_BASE_URL = "siteStreamBaseURL";
 
-
     public static final String ASYNC_NUM_THREADS = "async.numThreads";
+    public static final String CONTRIBUTING_TO = "contributingTo";
     public static final String ASYNC_DISPATCHER_IMPL = "async.dispatcherImpl";
     public static final String INCLUDE_RTS = "includeRTs";
     public static final String INCLUDE_ENTITIES = "includeEntities";
+    public static final String INCLUDE_MY_RETWEET = "includeMyRetweet";
+    public static final String LOGGER_FACTORY = "loggerFactory";
     public static final String JSON_STORE_ENABLED = "jsonStoreEnabled";
     public static final String MBEAN_ENABLED = "mbeanEnabled";
     public static final String STREAM_USER_REPLIES_ALL = "stream.user.repliesAll";
+    public static final String STREAM_STALL_WARNINGS_ENABLED = "stream.enableStallWarnings";
 
     public static final String MEDIA_PROVIDER = "media.provider";
     public static final String MEDIA_PROVIDER_API_KEY = "media.providerAPIKey";
@@ -113,18 +113,30 @@ public final class PropertyConfiguration extends ConfigurationBase implements ja
         // load from system properties
         try {
             props = (Properties) System.getProperties().clone();
+            try {
+                Map<String, String> envMap = System.getenv();
+                for(String key :envMap.keySet()){
+                    props.setProperty(key, envMap.get(key));
+                }
+            }catch(SecurityException ignore){}
             normalize(props);
         } catch (SecurityException ignore) {
             // Unsigned applets are not allowed to access System properties
             props = new Properties();
         }
         final String TWITTER4J_PROPERTIES = "twitter4j.properties";
-        // override System properties with ./twiter4j.properties in the classpath
+        // override System properties with ./twitter4j.properties in the classpath
         loadProperties(props, "." + File.separatorChar + TWITTER4J_PROPERTIES);
-        // then, override with /twiter4j.properties in the classpath
+        // then, override with /twitter4j.properties in the classpath
         loadProperties(props, Configuration.class.getResourceAsStream("/" + TWITTER4J_PROPERTIES));
-        // then, override with /WEB/INF/twiter4j.properties in the classpath
+        // then, override with /WEB/INF/twitter4j.properties in the classpath
         loadProperties(props, Configuration.class.getResourceAsStream("/WEB-INF/" + TWITTER4J_PROPERTIES));
+        // for Google App Engine
+        try {
+            loadProperties(props, new FileInputStream("WEB-INF/" + TWITTER4J_PROPERTIES));
+        } catch (SecurityException ignore) {
+        } catch (FileNotFoundException ignore) {
+        }
 
         setFieldsWithTreePath(props, treePath);
     }
@@ -137,7 +149,7 @@ public final class PropertyConfiguration extends ConfigurationBase implements ja
     }
 
     private boolean notNull(Properties props, String prefix, String name) {
-        return null != props.getProperty(prefix + name);
+        return props.getProperty(prefix + name) != null;
     }
 
     private boolean loadProperties(Properties props, String path) {
@@ -153,7 +165,7 @@ public final class PropertyConfiguration extends ConfigurationBase implements ja
         } catch (Exception ignore) {
         } finally {
             try {
-                if (null != fis) {
+                if (fis != null) {
                     fis.close();
                 }
             } catch (IOException ignore) {
@@ -201,7 +213,7 @@ public final class PropertyConfiguration extends ConfigurationBase implements ja
      */
     private void setFieldsWithTreePath(Properties props, String treePath) {
         setFieldsWithPrefix(props, "");
-        String[] splitArray = T4JInternalStringUtil.split(treePath, "/");
+        String[] splitArray = z_T4JInternalStringUtil.split(treePath, "/");
         String prefix = null;
         for (String split : splitArray) {
             if (!"".equals(split)) {
@@ -287,6 +299,9 @@ public final class PropertyConfiguration extends ConfigurationBase implements ja
         if (notNull(props, prefix, ASYNC_NUM_THREADS)) {
             setAsyncNumThreads(getIntProperty(props, prefix, ASYNC_NUM_THREADS));
         }
+        if (notNull(props, prefix, CONTRIBUTING_TO)) {
+            setContributingTo(getLongProperty(props, prefix, CONTRIBUTING_TO));
+        }
         if (notNull(props, prefix, ASYNC_DISPATCHER_IMPL)) {
             setDispatcherImpl(getString(props, prefix, ASYNC_DISPATCHER_IMPL));
         }
@@ -320,10 +335,6 @@ public final class PropertyConfiguration extends ConfigurationBase implements ja
             setRestBaseURL(getString(props, prefix, REST_BASE_URL));
         }
 
-        if (notNull(props, prefix, SEARCH_BASE_URL)) {
-            setSearchBaseURL(getString(props, prefix, SEARCH_BASE_URL));
-        }
-
         if (notNull(props, prefix, STREAM_BASE_URL)) {
             setStreamBaseURL(getString(props, prefix, STREAM_BASE_URL));
         }
@@ -339,6 +350,12 @@ public final class PropertyConfiguration extends ConfigurationBase implements ja
         if (notNull(props, prefix, INCLUDE_ENTITIES)) {
             setIncludeEntitiesEnbled(getBoolean(props, prefix, INCLUDE_ENTITIES));
         }
+        if (notNull(props, prefix, INCLUDE_MY_RETWEET)) {
+            setIncludeMyRetweetEnabled(getBoolean(props, prefix, INCLUDE_MY_RETWEET));
+        }
+        if (notNull(props, prefix, LOGGER_FACTORY)) {
+            setLoggerFactory(getString(props, prefix, LOGGER_FACTORY));
+        }
         if (notNull(props, prefix, JSON_STORE_ENABLED)) {
             setJSONStoreEnabled(getBoolean(props, prefix, JSON_STORE_ENABLED));
         }
@@ -348,6 +365,9 @@ public final class PropertyConfiguration extends ConfigurationBase implements ja
         if (notNull(props, prefix, STREAM_USER_REPLIES_ALL)) {
             setUserStreamRepliesAllEnabled(getBoolean(props, prefix, STREAM_USER_REPLIES_ALL));
         }
+        if (notNull(props, prefix, STREAM_STALL_WARNINGS_ENABLED)) {
+            setStallWarningsEnabled(getBoolean(props, prefix, STREAM_STALL_WARNINGS_ENABLED));
+        }
         if (notNull(props, prefix, MEDIA_PROVIDER)) {
             setMediaProvider(getString(props, prefix, MEDIA_PROVIDER));
         }
@@ -355,14 +375,15 @@ public final class PropertyConfiguration extends ConfigurationBase implements ja
             setMediaProviderAPIKey(getString(props, prefix, MEDIA_PROVIDER_API_KEY));
         }
         if (notNull(props, prefix, MEDIA_PROVIDER_PARAMETERS)) {
-            String[] propsAry = T4JInternalStringUtil.split(getString(props, prefix, MEDIA_PROVIDER_PARAMETERS), "&");
+            String[] propsAry = z_T4JInternalStringUtil.split(getString(props, prefix, MEDIA_PROVIDER_PARAMETERS), "&");
             Properties p = new Properties();
             for (String str : propsAry) {
-                String[] parameter = T4JInternalStringUtil.split(str, "=");
+                String[] parameter = z_T4JInternalStringUtil.split(str, "=");
                 p.setProperty(parameter[0], parameter[1]);
             }
             setMediaProviderParameters(p);
         }
+        cacheInstance();
     }
 
     protected boolean getBoolean(Properties props, String prefix, String name) {
@@ -379,7 +400,21 @@ public final class PropertyConfiguration extends ConfigurationBase implements ja
         }
     }
 
+    protected long getLongProperty(Properties props, String prefix, String name) {
+        String value = props.getProperty(prefix + name);
+        try {
+            return Long.parseLong(value);
+        } catch (NumberFormatException nfe) {
+            return -1L;
+        }
+    }
+
     protected String getString(Properties props, String prefix, String name) {
         return props.getProperty(prefix + name);
     }
+
+    // assures equality after deserialization
+    protected Object readResolve() throws ObjectStreamException {
+        return super.readResolve();
+    }
 }
