@@ -23,6 +23,7 @@ import twitter4j.internal.org.json.JSONObject;
 
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
 import java.util.HashMap;
 import java.util.Map;
 
@@ -37,14 +38,13 @@ public final class DataObjectFactory {
 
     private static final Constructor<Status> statusConstructor;
     private static final Constructor<User> userConstructor;
-    private static final Constructor<Tweet> tweetConstructor;
     private static final Constructor<Relationship> relationshipConstructor;
     private static final Constructor<Place> placeConstructor;
     private static final Constructor<SavedSearch> savedSearchConstructor;
     private static final Constructor<Trend> trendConstructor;
     private static final Constructor<Trends> trendsConstructor;
     private static final Constructor<IDs> IDsConstructor;
-    private static final Constructor<RateLimitStatus> rateLimitStatusConstructor;
+    private static final Method rateLimitStatusConstructor;
     private static final Constructor<Category> categoryConstructor;
     private static final Constructor<DirectMessage> directMessageConstructor;
     private static final Constructor<Location> locationConstructor;
@@ -52,59 +52,59 @@ public final class DataObjectFactory {
     private static final Constructor<RelatedResults> relatedResultsConstructor;
     private static final Constructor<StatusDeletionNotice> statusDeletionNoticeConstructor;
     private static final Constructor<AccountTotals> accountTotalsConstructor;
+    private static final Constructor<OEmbed> oembedConstructor;
 
     static {
         try {
-            statusConstructor = (Constructor<Status>) Class.forName("twitter4j.StatusJSONImpl").getDeclaredConstructor(JSONObject.class);
+            statusConstructor = (Constructor<Status>) Class.forName("twitter4j.internal.json.StatusJSONImpl").getDeclaredConstructor(JSONObject.class);
             statusConstructor.setAccessible(true);
 
-            userConstructor = (Constructor<User>) Class.forName("twitter4j.UserJSONImpl").getDeclaredConstructor(JSONObject.class);
+            userConstructor = (Constructor<User>) Class.forName("twitter4j.internal.json.UserJSONImpl").getDeclaredConstructor(JSONObject.class);
             userConstructor.setAccessible(true);
 
-            tweetConstructor = (Constructor<Tweet>) Class.forName("twitter4j.TweetJSONImpl").getDeclaredConstructor(JSONObject.class);
-            tweetConstructor.setAccessible(true);
-
-            relationshipConstructor = (Constructor<Relationship>) Class.forName("twitter4j.RelationshipJSONImpl").getDeclaredConstructor(JSONObject.class);
+            relationshipConstructor = (Constructor<Relationship>) Class.forName("twitter4j.internal.json.RelationshipJSONImpl").getDeclaredConstructor(JSONObject.class);
             relationshipConstructor.setAccessible(true);
 
-            placeConstructor = (Constructor<Place>) Class.forName("twitter4j.PlaceJSONImpl").getDeclaredConstructor(JSONObject.class);
+            placeConstructor = (Constructor<Place>) Class.forName("twitter4j.internal.json.PlaceJSONImpl").getDeclaredConstructor(JSONObject.class);
             placeConstructor.setAccessible(true);
 
-            savedSearchConstructor = (Constructor<SavedSearch>) Class.forName("twitter4j.SavedSearchJSONImpl").getDeclaredConstructor(JSONObject.class);
+            savedSearchConstructor = (Constructor<SavedSearch>) Class.forName("twitter4j.internal.json.SavedSearchJSONImpl").getDeclaredConstructor(JSONObject.class);
             savedSearchConstructor.setAccessible(true);
 
-            trendConstructor = (Constructor<Trend>) Class.forName("twitter4j.TrendJSONImpl").getDeclaredConstructor(JSONObject.class);
+            trendConstructor = (Constructor<Trend>) Class.forName("twitter4j.internal.json.TrendJSONImpl").getDeclaredConstructor(JSONObject.class);
             trendConstructor.setAccessible(true);
 
-            trendsConstructor = (Constructor<Trends>) Class.forName("twitter4j.TrendsJSONImpl").getDeclaredConstructor(String.class);
+            trendsConstructor = (Constructor<Trends>) Class.forName("twitter4j.internal.json.TrendsJSONImpl").getDeclaredConstructor(String.class);
             trendsConstructor.setAccessible(true);
 
-            IDsConstructor = (Constructor<IDs>) Class.forName("twitter4j.IDsJSONImpl").getDeclaredConstructor(String.class);
+            IDsConstructor = (Constructor<IDs>) Class.forName("twitter4j.internal.json.IDsJSONImpl").getDeclaredConstructor(String.class);
             IDsConstructor.setAccessible(true);
 
-            rateLimitStatusConstructor = (Constructor<RateLimitStatus>) Class.forName("twitter4j.RateLimitStatusJSONImpl").getDeclaredConstructor(JSONObject.class);
+            rateLimitStatusConstructor = Class.forName("twitter4j.internal.json.RateLimitStatusJSONImpl").getDeclaredMethod("createRateLimitStatuses", JSONObject.class);
             rateLimitStatusConstructor.setAccessible(true);
 
-            categoryConstructor = (Constructor<Category>) Class.forName("twitter4j.CategoryJSONImpl").getDeclaredConstructor(JSONObject.class);
+            categoryConstructor = (Constructor<Category>) Class.forName("twitter4j.internal.json.CategoryJSONImpl").getDeclaredConstructor(JSONObject.class);
             categoryConstructor.setAccessible(true);
 
-            directMessageConstructor = (Constructor<DirectMessage>) Class.forName("twitter4j.DirectMessageJSONImpl").getDeclaredConstructor(JSONObject.class);
+            directMessageConstructor = (Constructor<DirectMessage>) Class.forName("twitter4j.internal.json.DirectMessageJSONImpl").getDeclaredConstructor(JSONObject.class);
             directMessageConstructor.setAccessible(true);
 
-            locationConstructor = (Constructor<Location>) Class.forName("twitter4j.LocationJSONImpl").getDeclaredConstructor(JSONObject.class);
+            locationConstructor = (Constructor<Location>) Class.forName("twitter4j.internal.json.LocationJSONImpl").getDeclaredConstructor(JSONObject.class);
             locationConstructor.setAccessible(true);
 
-            userListConstructor = (Constructor<UserList>) Class.forName("twitter4j.UserListJSONImpl").getDeclaredConstructor(JSONObject.class);
+            userListConstructor = (Constructor<UserList>) Class.forName("twitter4j.internal.json.UserListJSONImpl").getDeclaredConstructor(JSONObject.class);
             userListConstructor.setAccessible(true);
 
-            relatedResultsConstructor = (Constructor<RelatedResults>) Class.forName("twitter4j.RelatedResultsJSONImpl").getDeclaredConstructor(JSONArray.class);
+            relatedResultsConstructor = (Constructor<RelatedResults>) Class.forName("twitter4j.internal.json.RelatedResultsJSONImpl").getDeclaredConstructor(JSONArray.class);
             relatedResultsConstructor.setAccessible(true);
 
             statusDeletionNoticeConstructor = (Constructor<StatusDeletionNotice>) Class.forName("twitter4j.StatusDeletionNoticeImpl").getDeclaredConstructor(JSONObject.class);
             statusDeletionNoticeConstructor.setAccessible(true);
 
-            accountTotalsConstructor = (Constructor<AccountTotals>) Class.forName("twitter4j.AccountTotalsJSONImpl").getDeclaredConstructor(JSONObject.class);
+            accountTotalsConstructor = (Constructor<AccountTotals>) Class.forName("twitter4j.internal.json.AccountTotalsJSONImpl").getDeclaredConstructor(JSONObject.class);
             accountTotalsConstructor.setAccessible(true);
+            oembedConstructor = (Constructor<OEmbed>) Class.forName("twitter4j.internal.json.OEmbedJSONImpl").getDeclaredConstructor(JSONObject.class);
+            oembedConstructor.setAccessible(true);
         } catch (NoSuchMethodException e) {
             throw new ExceptionInInitializerError(e);
         } catch (ClassNotFoundException e) {
@@ -208,29 +208,6 @@ public final class DataObjectFactory {
         }
     }
 
-    /**
-     * Constructs a Tweet object from rawJSON string.
-     *
-     * @param rawJSON raw JSON form as String
-     * @return Tweet
-     * @throws TwitterException when provided string is not a valid JSON string.
-     * @since Twitter4J 2.1.7
-     */
-    public static Tweet createTweet(String rawJSON) throws TwitterException {
-        try {
-            JSONObject json = new JSONObject(rawJSON);
-            return tweetConstructor.newInstance(json);
-        } catch (InstantiationException e) {
-            throw new TwitterException(e);
-        } catch (IllegalAccessException e) {
-            throw new AssertionError(e);
-        } catch (InvocationTargetException e) {
-            throw new TwitterException(e);
-        } catch (JSONException e) {
-            throw new TwitterException(e);
-        }
-    }
-
     /**
      * Constructs a Relationship object from rawJSON string.
      *
@@ -371,11 +348,11 @@ public final class DataObjectFactory {
      * @throws TwitterException when provided string is not a valid JSON string.
      * @since Twitter4J 2.1.7
      */
-    public static RateLimitStatus createRateLimitStatus(String rawJSON) throws TwitterException {
+    public static Map<String, RateLimitStatus> createRateLimitStatus(String rawJSON) throws TwitterException {
         try {
             JSONObject json = new JSONObject(rawJSON);
-            return rateLimitStatusConstructor.newInstance(json);
-        } catch (InstantiationException e) {
+            return (Map<String, RateLimitStatus>) rateLimitStatusConstructor.invoke(Class.forName("twitter4j.internal.json.RateLimitStatusJSONImpl"), json);
+        } catch (ClassNotFoundException e) {
             throw new TwitterException(e);
         } catch (IllegalAccessException e) {
             throw new AssertionError(e);
@@ -501,6 +478,29 @@ public final class DataObjectFactory {
         }
     }
 
+    /**
+     * Constructs an OEmbed object from rawJSON string.
+     *
+     * @param rawJSON raw JSON form as String
+     * @return OEmbed
+     * @throws TwitterException when provided string is not a valid JSON string.
+     * @since Twitter4J 3.0.2
+     */
+    public static OEmbed createOEmbed(String rawJSON) throws TwitterException {
+        try {
+            JSONObject json = new JSONObject(rawJSON);
+            return oembedConstructor.newInstance(json);
+        } catch (InstantiationException e) {
+            throw new TwitterException(e);
+        } catch (IllegalAccessException e) {
+            throw new AssertionError(e);
+        } catch (InvocationTargetException e) {
+            throw new TwitterException(e);
+        } catch (JSONException e) {
+            throw new TwitterException(e);
+        }
+    }
+
     /**
      * Construct an object from rawJSON string.  This method may be called
      * when you do not know what a given raw JSON string contains.  It will
@@ -523,26 +523,25 @@ public final class DataObjectFactory {
     public static Object createObject(String rawJSON) throws TwitterException {
         try {
             JSONObject json = new JSONObject(rawJSON);
-            JSONObjectType jsonObjectType = JSONObjectType.determine(json);
-            if (JSONObjectType.SENDER == jsonObjectType) {
-                return registerJSONObject(directMessageConstructor.newInstance(json.getJSONObject("direct_message")), json);
-            } else if (JSONObjectType.STATUS == jsonObjectType) {
-                return registerJSONObject(statusConstructor.newInstance(json), json);
-            } else if (JSONObjectType.DIRECT_MESSAGE == jsonObjectType) {
-                return registerJSONObject(directMessageConstructor.newInstance(json.getJSONObject("direct_message")), json);
-            } else if (JSONObjectType.DELETE == jsonObjectType) {
-                return registerJSONObject(statusDeletionNoticeConstructor.newInstance(json.getJSONObject("delete").getJSONObject("status")), json);
-            } else if (JSONObjectType.LIMIT == jsonObjectType) {
-                // TODO: Perhaps there should be a TrackLimitationNotice object?
-                // The onTrackLimitationNotice method could take that as an arg.
-                return json;
-            } else if (JSONObjectType.SCRUB_GEO == jsonObjectType) {
-                // TODO: Perhaps there should be a ScrubGeo object?
-                // The onScrubGeo method could take that as an arg.
-                return json;
-            } else {
-                // The object type is unrecognized...just return the json
-                return json;
+            JSONObjectType.Type jsonObjectType = JSONObjectType.determine(json);
+            switch (jsonObjectType) {
+                case SENDER:
+                    return registerJSONObject(directMessageConstructor.newInstance(json.getJSONObject("direct_message")), json);
+                case STATUS:
+                    return registerJSONObject(statusConstructor.newInstance(json), json);
+                case DIRECT_MESSAGE:
+                    return registerJSONObject(directMessageConstructor.newInstance(json.getJSONObject("direct_message")), json);
+                case DELETE:
+                    return registerJSONObject(statusDeletionNoticeConstructor.newInstance(json.getJSONObject("delete").getJSONObject("status")), json);
+                case LIMIT:
+                    // TODO: Perhaps there should be a TrackLimitationNotice object?
+                    // The onTrackLimitationNotice method could take that as an arg.
+                    return json;
+                case SCRUB_GEO:
+                    return json;
+                default:
+                    // The object type is unrecognized...just return the json
+                    return json;
             }
         } catch (InstantiationException e) {
             throw new TwitterException(e);
